import {Md5} from "ts-md5";
import Cookies from 'js-cookie';

export module OpenSubsonic {
    export let username = "";
    export let password = "";
    let token = "";
    let salt = "";
    export let base = "https://music.neshweb.net";
    const apiVer = "1.16.1"; // Version supported by Navidrome. Variable for easier updating
    const clientName = "Lydstyrke";

    export async function get(path: string, parameters: {parameter: string, value: string}[] = []) {
        const apiPath = getApiUrl(path, parameters);
        const res = (await fetch(apiPath));
        if (res.ok) {
            switch (res.headers.get("content-type")) {
                case("application/json"): {
                    return (await res.json())["subsonic-response"];
                }
                case("audio/mp4"): {
                    return res;
                }
            }
        }
        else {
            return false;
        }
    }

    export function getApiUrl(path: string, parameters: {parameter: string, value: string}[] = []) {
        let apiPath = generateBasePath(path);
        parameters.forEach(({parameter, value}) => {
            apiPath = apiPath + `&${parameter}=${value}`;
        })

        return apiPath;
    }

    const generateBasePath = (path: string) => {
        if (username === "") {
            const cookie = Cookies.get("subsonicUsername");
            if (typeof cookie !== "undefined") {
                username = cookie;
            }
        }

        let cookie = Cookies.get("subsonicToken");
        if (typeof cookie !== "undefined") {
            token = cookie;

            cookie = Cookies.get("subsonicSalt");
            if (typeof cookie !== "undefined") {
                salt = cookie;
            }
        }
        else {
            generateToken();
        }

        return `${base}/rest/${path}?u=${username}&s=${salt}&t=${token}&v=${apiVer}&c=${clientName}&f=json`;
    }

    const generateToken = () => {
        salt = "staticfornow";
        token = Md5.hashStr(password + salt);
    }

    export function storeCredentials() {
        const options = {
            expires: 7,
            path: "/",
            sameSite: "strict",
        };
        Cookies.set("subsonicToken", token, options);
        Cookies.set("subsonicSalt", salt, options);
        Cookies.set("subsonicUsername", username, options);
    }
}